package geniusweb.events;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import tudelft.utilities.junit.GeneralTests;

public class TournamentStartedTest extends GeneralTests<TournamentStarted> {

	private final ObjectMapper jackson = new ObjectMapper();

	private static final Long SESSIONNR = 1l;
	protected static final Long NOW = (long) 101;
	protected static final Long NOW1 = NOW + 1;
	private static final Long NSESSIONS = 5l;

	private static TournamentStarted tournamentStarted = new TournamentStarted(
			NSESSIONS, NOW);
	private static TournamentStarted tournamentStarteda = new TournamentStarted(
			NSESSIONS, NOW);
	private TournamentStarted otherTournamentStarted = new TournamentStarted(
			NSESSIONS + 1, NOW);
	private TournamentStarted tournamentStartedLater = new TournamentStarted(
			NSESSIONS, NOW1);

	private String tournamentstartedstring = "{\"tournamentstarted\":{\"time\":101,\"numberOfSessions\":5}}";

	@Override
	public List<List<TournamentStarted>> getGeneralTestData() {
		List<List<TournamentStarted>> list = new LinkedList<>();
		list.add(Arrays.asList(tournamentStarted, tournamentStarteda));
		list.add(Arrays.asList(otherTournamentStarted));
		list.add(Arrays.asList(tournamentStartedLater));
		return list;
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"TournamentStarted\\[.*" + NOW + ".*" + NSESSIONS + ".*\\]",
				"TournamentStarted\\[.*" + NOW + ".*" + (NSESSIONS + 1)
						+ ".*\\]",
				"TournamentStarted\\[.*" + NOW1 + ".*" + NSESSIONS + ".*\\]");
	}

	@Test
	public void serializeTournamentStartedTest()
			throws JsonProcessingException {
		System.out.println(jackson.writeValueAsString(tournamentStarted));
		assertEquals(tournamentstartedstring,
				jackson.writeValueAsString(tournamentStarted));
	}

	@Test
	public void deserializeTournamentStartedTest() throws IOException {
		NegotiationEvent evt = jackson.readValue(tournamentstartedstring,
				NegotiationEvent.class);
		assertEquals(TournamentStarted.class, evt.getClass());
		TournamentStarted event = (TournamentStarted) evt;
		// compare fields, as sessionended is a derived/new inner class
		assertEquals(NSESSIONS, event.getNumberOfSessions());
		assertEquals(NOW, event.getTime());
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullTime() {
		new TournamentStarted(NSESSIONS, null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNegativeTime() {
		new TournamentStarted(NSESSIONS, -1l);
	}

}
